@@ -9,7 +9,7 @@ def main():
     parser.add_argument('dataset_location', nargs=1, help= "Defines the location of the dataset to be checked.")
     parser.add_argument('--test', '-t', '--test=', '-t=', help= "Select the Checks you want to perform.  Either all (default), cf, ioos, or acdd.", nargs='+', default=[], choices=ComplianceCheckerCheckSuite.checkers.keys())
     parser.add_argument('--criteria', '-c', help="Define the criteria for the checks.  Either Strict, Normal, or Lenient.  Defaults to Normal.", nargs='?', default='normal', choices = ['lenient', 'normal', 'strict'])
-    parser.add_argument('--verbose' , '-v', help="Increase Output Verbosity", action="count")
+    parser.add_argument('--verbose' , '-v', help="Increase output. May be specified up to three times.", action="count")
 
     args = parser.parse_args()
 
