@@ -2,6 +2,7 @@
 Compliance Checker suite runner
 """
 
+import sys
 import inspect
 import itertools
 from netCDF4 import Dataset
@@ -11,6 +12,7 @@ from owslib.sos import SensorObservationService
 from owslib.swe.sensor.sml import SensorML
 from urlparse import urlparse
 import requests
+import textwrap
 
 class CheckSuite(object):
 
@@ -30,9 +32,9 @@ class CheckSuite(object):
         val = check_method(ds)
 
         if isinstance(val, list):
-            return [fix_return_value(v, check_method.im_func.func_name) for v in val]
+            return [fix_return_value(v, check_method.im_func.func_name, check_method, check_method.im_self) for v in val]
 
-        return [fix_return_value(val, check_method.im_func.func_name)]
+        return [fix_return_value(val, check_method.im_func.func_name, check_method, check_method.im_self)]
 
     def _get_valid_checkers(self, ds, checker_names):
         """
@@ -65,11 +67,10 @@ class CheckSuite(object):
         """
         Runs this CheckSuite on the dataset with all the passed Checker instances.
 
-        Returns a dictionary mapping checker names to their grouped scores.
+        Returns a dictionary mapping checker names to a 2-tuple of their grouped scores and errors/exceptions while running checks.
         """
 
         ret_val      = {}
-        check_number = 0
         fail_flag    = False
 
         checkers     = self._get_valid_checkers(ds, checker_names)
@@ -79,30 +80,47 @@ class CheckSuite(object):
 
         for checker_name, checker_class in checkers:
 
-            checker = checker_class()   # @TODO: combine with load_datapair/setup
-            dsp = checker.load_datapair(ds)
+            checker            = checker_class()   # @TODO: combine with load_datapair/setup
+            dsp                = checker.load_datapair(ds)
             checker.setup(dsp)
-            checks = self._get_checks(checker)
 
-            vals = list(itertools.chain.from_iterable(map(lambda c: self._run_check(c, dsp), checks)))
+            checks             = self._get_checks(checker)
+            vals               = []
+            errs               = {}   # check method name -> (exc, traceback)
+
+            for c in checks:
+                try:
+                    vals.extend(self._run_check(c, dsp))
+                except Exception as e:
+                    errs[c.im_func.func_name] = (e, sys.exc_info()[2])
+
+            # score the results we got back
             groups = self.scores(vals)
 
-            ret_val[checker_name] = groups
+            ret_val[checker_name] = groups, errs
 
         return ret_val
 
-    def standard_output(self, criteria, check_name, groups):
-        '''
+    @classmethod
+    def passtree(cls, groups, limit):
+        for r in groups:
+            if r.children:
+                x = cls.passtree(r.children, limit)
+                if r.weight >= limit and x == False:
+                    return False
+
+            if r.weight >= limit and r.value[0] != r.value[1]:
+                return False
+
+        return True
+
+    def standard_output(self, limit, check_name, groups):
+        """
         Generates the Terminal Output for Standard cases
 
         Returns the dataset needed for the verbose output, as well as the failure flags.
-        '''
-        if criteria == 'normal':
-            limit = 2
-        elif criteria == 'strict':
-            limit = 1
-        elif criteria == 'lenient':
-            limit = 3
+        """
+
 
         score_list = []
         score_only_list= []
@@ -122,54 +140,99 @@ class CheckSuite(object):
 
         fail_flag = 0
 
+        fail_flag = limit
+        print '\n'
+        print "-"*80
+        print '{:^80}'.format("The dataset scored %r out of %r points" % (points, out_of))
+        print '{:^80}'.format("during the %s check" % check_name)
+        print "-"*80
+
+        return  [score_list, points, out_of]
+
+    def non_verbose_output_generation(self, score_list, groups, limit, points, out_of):
+
         if points < out_of:
-            fail_flag = limit
+            print '{:^80}'.format("Scoring Breakdown:")
             print '\n'
-            print "-"*55
-            print "   The dataset scored %r out of %r required points" % (points, out_of)
-            print "            during the %s check" % check_name
-            print "      This test has passed under %s critera" % criteria
-            print "-"*55
-        return [score_list, fail_flag, limit]
+            priority_flag = 3
+            for x in range(len(score_list)):
+                if score_list[x][1] == 3 and limit <= 3 :
+                    if priority_flag ==3:
+                        print '{:^80}'.format("High Priority")
+                        print "-"*80
+                        print '%-36s:%8s:%6s' % ('    Name', 'Priority', 'Score')
+                        priority_flag -= 1
+                    print '%-40s:%s:%6s/%1s'  % (score_list[x][0][0:39], score_list[x][1], score_list[x][2][0], score_list[x][2][1])
+                
+                elif score_list[x][1] == 2 and limit <= 2 :
+                    if priority_flag ==2:
+                        print '\n'
+                        print '{:^80}'.format("Medium Priority")
+                        print "-"*80
+                        print '%-36s:%8s:%6s' % ('    Name', 'Priority', 'Score')
+                        priority_flag -= 1
+                    print '%-40s:%s:%6s/%1s'  % (score_list[x][0][0:39], score_list[x][1], score_list[x][2][0], score_list[x][2][1])
+                    
+                elif score_list[x][1] == 1 and limit == 1 :
+                    if priority_flag ==1:
+                        print '\n'
+                        print '{:^80}'.format("Low Priority")
+                        print "-"*80
+                        print '%-36s:%8s:%6s' % ('    Name', 'Priority', 'Score')
+                        priority_flag -= 1
+                    print '%-40s:%s:%6s/%1s'  % (score_list[x][0][0:39], score_list[x][1], score_list[x][2][0], score_list[x][2][1])
+                    
+                elif score_list[x][1] == 1 and limit == 1 and priority_flag == 2:
+                    print '{:^80}'.format('No medium priority tests present')
+                    print '-'*80
+                    priority_flag -= 1
+            #Catch All for pretty presentation
+            if priority_flag == 2 and limit == 2:
+                print '{:^80}'.format('No Medium priority tests present')
+                print '-'*80
+
+            if priority_flag == 2 and limit == 1:
+                print '{:^80}'.format('No Medium priority tests present')
+                print '-'*80
+                print ''
+                print '{:^80}'.format('No Low priority tests present')
+                print '-'*80
+
+            if priority_flag == 1 and limit == 1:
+                print '{:^80}'.format('No Low priority tests present')
+                print '-'*80
+
+            print "\n"+"\n"+'-'*80
+            print '{:^80}'.format('Reasoning for the failed tests given below:')
+            print '\n'
+            print '%s%37s:%10s:%8s' % ('Name','Priority', '  Score', 'Reasoning')
+            print "-"*80
+            self.reasoning_routine(groups, 0)
+
+        else: 
+            print "All tests passed!"
 
 
-    def verbose_output_generation(self, groups, verbose, score_list, limit):
+
+    def verbose_output_generation(self, groups, limit, points, out_of):
         '''
         Generates the Terminal Output for Verbose cases
         '''
-        sub_tests = []
-        if verbose == 1:
-            print "\n"+"-"*55
-            print "The following tests failed:" 
-            priority_flag = 3
-
-            for x in range(len(score_list)):
-                if score_list[x][1] == 3 and limit <= 3 and priority_flag == 3:
-                    print '----High priority tests failed-----'
-                    print '%-36s:%8s:%6s' % ('    Name', 'Priority', 'Score')
-                    priority_flag -= 1
-                elif score_list[x][1] == 2 and limit <= 2 and priority_flag == 2:
-                    print '----Medium priority tests failed-----'
-                    print '%-36s:%8s:%6s' % ('    Name', 'Priority', 'Score')
-                    priority_flag -=1
-                elif score_list[x][1] == 1 and limit <= 1 and priority_flag == 1:
-                    print '----Low priority tests failed-----'
-                    print '%-36s:%8s:%6s' % ('    Name', 'Priority', 'Score')
-                    priority_flag -= 1
-                if score_list[x][2][0] < score_list[x][2][1] and score_list[x][1] >= limit:
-                    print '%-40s:%s:%6s/%1s'  % (score_list[x][0], score_list[x][1], score_list[x][2][0], score_list[x][2][1])
-
-        if verbose >= 2:
-            print "Summary of all the checks performed:" 
-            
-            priority_flag = 3
-
-            self.print_routine(groups, 0, verbose, priority_flag)
+        priority_flag = 3
+        print '{:^80}'.format("Verbose Scoring Breakdown:"),
+        self.print_routine(groups, 0, priority_flag)
+        if points < out_of:
+            print "\n"+"\n"+'-'*80
+            print '{:^80}'.format('Reasoning for the failed tests given below:')
+            print '\n'
+            print '%s%37s:%10s:%8s' % ('Name','Priority', '  Score', 'Reasoning')
+            print "-"*80
+            self.reasoning_routine(groups, 0)
 
         pass
 
 
-    def print_routine(self, list_of_results, indent, verbose, priority_flag):
+    def print_routine(self, list_of_results, indent, priority_flag):
         """
         print routine performed
         """
@@ -183,34 +246,61 @@ class CheckSuite(object):
         grouped_sorted = []
         grouped_sorted = sorted(list_of_results, key=weight_func, reverse=True)
 
-
-        #Loop over inoput
+        #Loop over input
         for res in grouped_sorted:
             #If statements to print the proper Headings
             if res.weight == 3 and indent == 0 and priority_flag == 3:
-                print "\nHigh Priority"
-                print "-------------"
+                print '\n'
+                print '{:^80}'.format("High Priority")
+                print "-"*80
                 print '%-36s:%8s:%6s' % ('    Name', 'Priority', 'Score')
 
                 priority_flag -= 1
             if res.weight == 2 and indent == 0 and priority_flag == 2:
-                print "\nMedium Priority"
-                print "---------------"
+                print '\n'
+                print '{:^80}'.format("Medium Priority")
+                print "-"*80
                 print '%-36s:%8s:%6s' % ('    Name', 'Priority', 'Score')
 
                 priority_flag -= 1
             if res.weight ==1 and indent ==0 and priority_flag == 1:
-                print "\nLow Priority"
-                print "------------"
+                print '\n'
+                print '{:^80}'.format("Low Priority")
+                print "-"*80
                 print '%-36s:%8s:%6s' % ('    Name', 'Priority', 'Score')
                 priority_flag -= 1
 
 
-            print '%-40s:%s:%6s/%1s' % (indent*'    '+res.name, res.weight, res.value[0], res.value[1])
-            if res.children and verbose >1:
-                self.print_routine(res.children, indent+1, verbose-1, priority_flag)
+            print '%-40s:%s:%s%6s/%1s' % ((indent*'    '+res.name)[0:39], res.weight, indent*'  ', res.value[0], res.value[1])
+            if res.children:
+                self.print_routine(res.children, indent+1, priority_flag)
 
 
+    def reasoning_routine(self, list_of_results, indent, line = True):
+        """
+        print routine performed
+        """
+        def weight_func(r):
+            """
+            Function that returns the weight, used for sorting by priority
+            """
+            return r.weight
+
+        #Sorting method used to properly sort the output by priority.
+        grouped_sorted = []
+        grouped_sorted = sorted(list_of_results, key=weight_func, reverse=True)
+
+        wrapper = textwrap.TextWrapper(initial_indent = '', width = 80, subsequent_indent = ' '*54)
+        for res in grouped_sorted:
+            if (res.value[0] != res.value[1]) and not res.msgs:
+                print '%-39s:%1s:%6s/%2s : %s' %(str(indent*'    '+res.name)[0:39], res.weight, str(res.value[0]), str(res.value[1]), ' ')
+            
+            if (res.value[0] != res.value[1]) and res.msgs:
+                print wrapper.fill('%-39s:%1s:%6s/%2s : %s' %(str(indent*'    '+res.name)[0:39], res.weight, str(res.value[0]), str(res.value[1]), str("\n".join(res.msgs))))
+
+            if res.children:
+                self.reasoning_routine(res.children, indent+1, False)
+
     def load_dataset(self, ds_str):
         """
         Helper method to load a dataset or SOS GC/DS url.
